package cn.com.ty.lift.business.common;

import cn.com.ty.lift.business.capital.dao.entity.CapitalRepair;
import cn.com.ty.lift.business.capital.dao.entity.CapitalRepairHistory;
import cn.com.ty.lift.business.capital.service.CapitalRepairHistoryService;
import cn.com.ty.lift.business.library.dao.entity.Lift;
import cn.com.ty.lift.business.library.dao.entity.LiftHistory;
import cn.com.ty.lift.business.library.service.LiftHistoryService;
import cn.com.ty.lift.business.project.dao.entity.Project;
import cn.com.ty.lift.business.project.dao.entity.ProjectHistory;
import cn.com.ty.lift.business.project.service.ProjectHistoryService;
import cn.com.ty.lift.common.constants.FieldName;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.lang.reflect.Field;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @author bieao
 * @date 2020/3/10
 * @description BeanUtil
 */
@Component
@Slf4j
public class BeanUtils {

    @Resource
    private ProjectHistoryService projectHistoryService;

    @Resource
    private LiftHistoryService liftHistoryService;

    @Resource
    private CapitalRepairHistoryService capitalRepairHistoryService;


    /**
     * @param oldEntry    旧实体
     * @param newEntry    新实体
     * @param mtCompanyId 公司id
     * @description 比较实体，并存入相应操作记录表
     * @date 2020/3/10 12:56 下午
     */
    public void saveOperaHistory(Object oldEntry, Object newEntry, Long mtCompanyId) throws NoSuchFieldException {
        Map<String, Object> oldResult = BeanUtil.beanToMap(oldEntry);
        oldResult.remove("createDate");
        oldResult.remove("updateDate");
        oldResult.remove("creatorId");
        oldResult.remove("updateId");
        Map<String, Object> newResult = BeanUtil.beanToMap(newEntry);
        newResult.remove("createDate");
        newResult.remove("updateDate");
        newResult.remove("creatorId");
        newResult.remove("updateId");
        List<ProjectHistory> projectHistories = new ArrayList<>();
        List<LiftHistory> liftHistories = new ArrayList<>();
        List<CapitalRepairHistory> capitalRepairHistories = new ArrayList<>();
        for (Map.Entry<String, Object> entry : oldResult.entrySet()) {
            Object oldValue = oldResult.get(entry.getKey());
            Object newValue = newResult.get(entry.getKey());
            if (Objects.isNull(oldValue) && Objects.isNull(newValue)) continue;
            if (Objects.isNull(newValue)) continue;
            Class<?> cls = newEntry.getClass();
            Field field = cls.getDeclaredField(entry.getKey());
            if (!field.isAnnotationPresent(FieldName.class)) continue;
            FieldName fieldName = field.getDeclaredAnnotation(FieldName.class);
            boolean result = ObjectUtil.notEqual(oldValue, newValue);
            if (result) {
                if (newEntry instanceof Project) {
                    Project project = (Project) newEntry;
                    ProjectHistory projectHistory = new ProjectHistory();
                    projectHistory.setProjectId(project.getId());
                    projectHistory.setMtCompanyId(mtCompanyId);
                    projectHistory.setOperaItem(fieldName.name());
                    projectHistory.setDescription("编辑项目信息");
                    projectHistory.setName(project.getProjectName());
                    projectHistory.setCode(project.getProjectCode());
                    projectHistory.setBeforeContent(oldValue.toString());
                    projectHistory.setAfterContent(newValue.toString());
                    projectHistory.setOperatorId(10000L);
                    projectHistory.setOperateDate(LocalDateTime.now());
                    projectHistories.add(projectHistory);
                } else if (newEntry instanceof Lift) {
                    Lift lift = (Lift) newEntry;
                    LiftHistory liftHistory = new LiftHistory();
                    liftHistory.setLiftId(lift.getId());
                    liftHistory.setCode(lift.getLiftCode());
                    liftHistory.setMtCompanyId(mtCompanyId);
                    liftHistory.setOperaItem(fieldName.name());
                    liftHistory.setDescription("编辑电梯信息");
                    liftHistory.setBeforeContent(oldValue.toString());
                    liftHistory.setAfterContent(newValue.toString());
                    liftHistory.setOperatorId(10000L);
                    liftHistory.setOperateDate(LocalDateTime.now());
                    liftHistories.add(liftHistory);
                } else if (newEntry instanceof CapitalRepair) {
                    CapitalRepair repair = (CapitalRepair) newEntry;
                    CapitalRepairHistory capitalRepairHistory = new CapitalRepairHistory();
                    capitalRepairHistory.setProjectId(repair.getId());
                    capitalRepairHistory.setProjectCode(repair.getProjectCode());
                    capitalRepairHistory.setMtCompanyId(mtCompanyId);
                    capitalRepairHistory.setOperaItem(fieldName.name());
                    capitalRepairHistory.setDescription("编辑大修信息");
                    capitalRepairHistory.setBeforeContent(oldValue.toString());
                    capitalRepairHistory.setAfterContent(newValue.toString());
                    capitalRepairHistory.setOperatorId(10000L);
                    capitalRepairHistory.setOperateDate(LocalDateTime.now());
                    capitalRepairHistories.add(capitalRepairHistory);
                }
            }
        }
        if (newEntry instanceof Project) {
            projectHistoryService.saveBatch(projectHistories, projectHistories.size());
            log.info("保存项目操作记录：" + projectHistories.size() + "条");
        } else if (newEntry instanceof Lift) {
            liftHistoryService.saveBatch(liftHistories, liftHistories.size());
            log.info("保存电梯操作记录：" + liftHistories.size() + "条");
        } else if (newEntry instanceof CapitalRepair) {
            capitalRepairHistoryService.saveBatch(capitalRepairHistories, capitalRepairHistories.size());
            log.info("保存大修项目操作记录：" + capitalRepairHistories.size() + "条");
        }
    }
}
